// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "AudioAsset.hpp"

using namespace Dawn;

AudioAsset::AudioAsset(AssetManager *man, std::string name) :
  Asset(man, name),
  loader(name + ".audio")
{

}

void AudioAsset::updateSync() {
  if(this->state != 0x07) return;

  // THIS WILL DEFINITELY CHANGE
  this->state = 0x08;
  this->loaded = true;
}

void AudioAsset::updateAsync() {
  if(this->state != 0x00) return;
  char *start, *end;
  char buffer[1024];

  // Open asset for reading
  this->state = 0x01;
  this->loader.open();

  // Parse header data.
  this->state = 0x02;
  this->loader.read((uint8_t *)buffer, 1024);

  // Channel count
  this->state = 0x03;
  start = buffer;
  end = strchr(start, '|');
  *end = '\0';
  this->channelCount = atoi(start);
  assertTrue(this->channelCount > 0, "AudioAsset::updateAsync: Channel count must be greater than 0");

  // Sample Rate
  this->state = 0x04;
  start = end + 1;
  end = strchr(start, '|');
  *end = '\0';
  this->sampleRate = (uint32_t)strtoul(start, NULL, 10);
  assertTrue(this->sampleRate > 0, "AudioAsset::updateAsync: Sample rate must be greater than 0");

  // Number of samples per channel
  this->state = 0x05;
  start = end + 1;
  end = strchr(start, '|');
  *end = '\0';
  this->samplesPerChannel = atoi(start);
  assertTrue(this->samplesPerChannel > 0, "AudioAsset::updateAsync: Samples per channel must be greater than 0");

  // Total Data Length
  this->state = 0x06;
  start = end + 1;
  end = strchr(start, '|');
  *end = '\0';
  this->bufferSize = (size_t)atoll(start);
  assertTrue(this->bufferSize > 0, "AudioAsset::updateAsync: Buffer size must be greater than 0");

  // Determine frame size
  this->frameSize = sizeof(int16_t) * this->channelCount;

  // Indicated start of data
  this->state = 0x07;
  this->bufferStart = end - buffer;
}