// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "display/shader/Shader.hpp"
#include "display/mesh/Mesh.hpp"

namespace Dawn {
  struct ShaderPassItem;

  struct ShaderPassItem {
    Shader *shader = nullptr;
    int32_t priority = 0;
    std::vector<struct ShaderPassItem>::iterator index;

    Mesh *mesh;
    int32_t start = 0;
    int32_t count = -1;
    float_t w = 0;
    renderflag_t renderFlags = RENDER_MANAGER_RENDER_FLAG_DEPTH_TEST;
    enum MeshDrawMode drawMode = MESH_DRAW_MODE_TRIANGLES;
    
    // Parameters
    std::map<shaderparameter_t, struct Color> colorValues;
    std::map<shaderparameter_t, bool_t> boolValues;
    std::map<shaderparameter_t, glm::mat4> matrixValues;
    std::map<shaderparameter_t, glm::vec3> vec3Values;
    std::map<shaderparameter_t, textureslot_t> textureValues;
    std::map<shaderparameter_t, float_t> floatValues;
    std::map<shaderbufferlocation_t, IShaderParameterBuffer<shaderbufferslot_t>*> parameterBuffers;

    // Textures
    std::map<textureslot_t, Texture*> textureSlots;
  };
}