// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "util/UsageLock.hpp"

using namespace Dawn;

UsageLock::UsageLock() : UsageLock(&this->internalPool) {
}

UsageLock::UsageLock(usagelockid_t *lockPool) {
  assertNotNull(lockPool, "Lock pool cannot be null");
  this->pool = lockPool;
  this->onEmpty = []() {};
  this->onFirstLock = []() {};
}

usagelockid_t UsageLock::createLock() {
  usagelockid_t lock = (*this->pool)++;
  this->locks.push_back(lock);
  if(this->locks.size() == 1) this->onFirstLock();
  return lock;
}

void UsageLock::removeLock(usagelockid_t lock) {
  auto it = std::find(this->locks.begin(), this->locks.end(), lock);
  if(it == this->locks.end()) return;
  this->locks.erase(it);
  this->onLockRemoved(lock);
  if(this->locks.size() == 0) this->onEmpty();
}