// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "dawnsharedlibs.hpp"
#include "util/memory.hpp"
#include "assert/assert.hpp"
#include "util/array.hpp"

namespace Dawn {
  enum XmlParseState {
    XML_PARSE_STATE_DOING_NOTHING,
    XML_PARSE_STATE_PARSING_TAG_NAME,
    XML_PARSE_STATE_LOOKING_FOR_ATTRIBUTE,
    XML_PARSE_STATE_PARSING_ATTRIBUTE_NAME,
    XML_PARSE_STATE_LOOKING_FOR_ATTRIBUTE_VALUE,
    XML_PARSE_STATE_PARSING_ATTRIBUTE_VALUE,
    XML_PARSE_STATE_PARSING_VALUE,
    XML_PARSE_STATE_PARSING_CLOSE,
    XML_PARSE_STATE_PARSING_COMMENT
  };

  class Xml;
  struct XmlNode;

  enum XmlNodeType {
    XML_NODE_TYPE_TEXT,
    XML_NODE_TYPE_ELEMENT
  };

  class Xml {
    protected:
      static bool_t isWhitespace(char_t c);

      // @deprecated
      std::vector<Xml*> children;

    public:
      static Xml load(std::string data);
      static void load(Xml *xml, std::string data, size_t *j);

      std::string node;
      std::string innerXml;
      std::string outerXml;
      std::string textContent;
      std::map<std::string, std::string> attributes;
      std::vector<struct XmlNode> childNodes;
      
      std::vector<Xml*> getChildrenOfType(std::string type);
      Xml * getFirstChildOfType(std::string type);

      ~Xml();
  };

  struct XmlNode {
    enum XmlNodeType nodeType;
    std::string value;
    Xml *child;

    XmlNode();
  };
}