// Copyright (c) 2022 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "memory.hpp"
#include "assert/assert.hpp"

void * memoryAllocate(const size_t size) {
  assertTrue(size >= 0, "memoryAllocate: size must be greater than 0 or equal to.");
  void *x = (void *)memoryCallMalloc(size);
  assertNotNull(x, "memoryAllocate: Failed to allocate memory");
  return x;
}

void * memoryAllocateEmpty(const size_t count, const size_t size) {
  assertTrue(count >= 0, "memoryAllocateEmpty: count must be greater than or equal to 0");
  assertTrue(size >= 0, "memoryAllocateEmpty: size must be greater than or equal to 0");
  void *x = (void *)memoryCallCalloc(count, size);
  
  assertNotNull(x, "memoryAllocateEmpty: Failed to allocate memory");
  return x;
}

void memoryFree(void *pointer) {
  memoryCallFree(pointer);
}

void * memoryReallocate(void *ptr, size_t newSize) {
  assertTrue(newSize >= 0, "memoryReallocate: newSize must be greater than or equal to 0");
  return memoryCallRealloc(ptr, newSize);
}

void memoryCopy(void *source, void *destination, size_t size) {
  assertNotNull(source, "memoryCopy: source must not be null");
  assertNotNull(destination, "memoryCopy: destination must not be null");
  assertTrue(destination != source, "memoryCopy: destination must not be source");
  assertTrue(size > 0, "memoryCopy: size must be greater than 0");
  memcpy(destination, source, size);
}

int32_t memoryCompare(const void *left, const void *right, const size_t size) {
  assertTrue(left != right, "memoryCompare: left must not be right");
  assertTrue(size > 0, "memoryCompare: size must be greater than 0");
  return memcmp(left, right, size);
}

void memorySet(void *dest, uint8_t data, size_t length) {
  assertNotNull(dest, "memorySet: dest must not be null");
  assertTrue(length > 0, "memorySet: length must be greater than 0");
  memset(dest, data, length);
}

void memoryInit() {

}

void memoryDispose() {
  
}